/****************************************************************************
*  Copyright (c) 2011 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  18.04.2011  mifi  First Version for the EIR
****************************************************************************/

OUTPUT_FORMAT("elf32-littlearm", "elf32-littlearm", "elf32-littlearm")
OUTPUT_ARCH(arm)
ENTRY(ResetHandler)

/*
 * Define exception stack size here.
 */
FIQ_STACK_SIZE   = 0;      /* FIQ        */
IRQ_STACK_SIZE   = 512;    /* IRQ        */
SVC_STACK_SIZE   = 0;      /* Supervisor */
ABORT_STACK_SIZE = 0;      /* Abort      */ 
UNDEF_STACK_SIZE = 0;      /* Undefined  */
SYS_STACK_SIZE   = 1024;   /* System     */


/*
 * Take a look in the "The GNU linker" manual, here you get
 * the following information about the "MEMORY":
 *
 * "The MEMORY command describes the location and size of 
 * blocks of memory in the target."
 */
MEMORY
{
   FLASH (rx)  : ORIGIN = 0x00100000, LENGTH = 512K
   IRAM  (rwx) : ORIGIN = 0x00200000, LENGTH = 64K
   RAM   (rwx) : ORIGIN = 0x20000000, LENGTH = 64M
}


/*
 * And the "SECTION" is used for:
 *
 * "The SECTIONS command tells the linker how to map input
 * sections into output sections, and how to place the output
 * sections in memory.
 */
SECTIONS
{

   /*
    * The ".text" section is used for the code, and
    * read only (.rodata) data. Even the vectors (.vectors)
    * MUST be saved at the start of this section.
    */
   .text :
   {
      _stext = .;          /* Provide the name for the start of this section */
      
      CREATE_OBJECT_SYMBOLS
      KEEP(*(.vectors))
      
      *(.text)
      *(.text.*)
            
      . = ALIGN(4);        /* Align the start of the rodata part */
      *(.rodata)
      *(.rodata.*)
      
      . = ALIGN(4);        /* Align the end of the section */
   } > FLASH = 0
   _etext = .;             /* Provide the name for the end of this section */
   
   
   /*
    * The ".data" section is used for initialized data
    * and for functions (.fastrun) which should be copied 
    * from flash to ram. This functions will later be
    * executed from ram instead of flash.
    */
   .data : AT (_etext)
   {
      . = ALIGN(4);        /* Align the start of the section */
      _sdata = .;          /* Provide the name for the start of this section */
      
      *(.data)
      *(.data.*)
      
      . = ALIGN(4);        /* Align the start of the fastrun part */
      *(.fastrun)
      *(.fastrun.*)
      
      . = ALIGN(4);        /* Align the end of the section */
   } > RAM
   _edata = .;             /* Provide the name for the end of this section */
   

   /*
    * The ".bss" section is used for uninitialized data.
    * This section will be cleared by the startup code.
    */
   .bss :
   {
      . = ALIGN(4);        /* Align the start of the section */
      _sbss = .;           /* Provide the name for the start of this section */
      
      *(.bss)
      *(.bss.*)
      
      . = ALIGN(4);        /* Align the end of the section */
   } > RAM
   _ebss = .;              /* Provide the name for the end of this section */
   
   
   /*
    * The ".stack" section is used for the exception stack.
    */
   .stack :
   {
      . = ALIGN(4);        /* Align the start of the section */
      _sstack = .;         /* Provide the name for the start of this section */

      . += FIQ_STACK_SIZE;
      . = ALIGN(4);
      _efiq_stack = .;     /* Provide end of FIQ stack */
      
      . += IRQ_STACK_SIZE;
      . = ALIGN(4);
      _eirq_stack = .;     /* Provide end of IRQ stack */
      
      . += SVC_STACK_SIZE;
      . = ALIGN(4);
      _esvc_stack = .;     /* Provide end of Supervisor stack */
      
      . += ABORT_STACK_SIZE;
      . = ALIGN(4);
      _eabort_stack = .;   /* Provide end of Abort stack */
      
      . += UNDEF_STACK_SIZE;
      . = ALIGN(4);
      _eundef_stack = .;   /* Provide end of Undefined stack */
      
      . += SYS_STACK_SIZE;
      . = ALIGN(4);
      _esys_stack = .;     /* Provide end of System stack */
   
   } > IRAM    
   _estack = .;            /* Provide the name for the end of this section */


   /*
    * The ".heap" section is used for the heap.
    */
   .heap :
   {  
      . = ALIGN(4);           
      __heap_start = .;
   } > RAM      
      
}

/*** EOF **/

