/****************************************************************************
*  Copyright (c) 2011 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  18.04.2011  mifi  First Version for the EIR board.
****************************************************************************/


#define ARM_MODE_USER   0x10
#define ARM_MODE_FIQ    0x11
#define ARM_MODE_IRQ    0x12
#define ARM_MODE_SVC    0x13
#define ARM_MODE_ABORT  0x17
#define ARM_MODE_UNDEF  0x1B
#define ARM_MODE_SYS    0x1F

#define ARM_CPSR_I_BIT  0x80
#define ARM_CPSR_F_BIT  0x40


/***************************************************************************/
/*  Exception table                                                        */
/***************************************************************************/
 
   .section .vectors, "ax",%progbits
   .global _vectors
_vectors:
   ldr   pc, ResetAddress
   ldr   pc, UndefAddress
   ldr   pc, SWIAddress
   ldr   pc, PAbortAddress
   ldr   pc, DAbortAddress
   ldr   pc, RESAddress
   
#if 0   
   /*
    * On IRQ the PC will be loaded from AIC_IVR, which
    * provides the address previously set in AIC_SVR.
    * The interrupt routine will be called in ARM_MODE_IRQ
    * with IRQ disabled and FIQ unchanged.
    */
   ldr   pc, [pc, #-0xF20]   /* Interrupt request, auto vectoring. */
   ldr   pc, [pc, #-0xF20]   /* Fast interrupt request, auto vectoring. */
#else   
   ldr   pc, IRQAddress
   ldr   pc, FIQAddress
#endif   
   
ResetAddress:
   .word ResetHandler
UndefAddress:
   .word UndefHandler
SWIAddress:
   .word SWIHandler
PAbortAddress:
   .word PAbortHandler
DAbortAddress:
   .word DAbortHandler
RESAddress:
   .word RESHandler
IRQAddress:
   .word IRQHandler
FIQAddress:
   .word FIQHandler
   
   
/***************************************************************************/
/*  Text area                                                              */
/***************************************************************************/
   
   .section .text, "ax", %progbits
   .global ResetHandler
   .align 4
   
ResetHandler: 

   /*
    * Set the stack pointer for temporary use.
    * The stack will be set later to the correct value.
    */ 
   ldr   sp, =_estack         
  
   
   /*
    * Hardware initialization
    * E.g. disabling a watchdog, setup the PLL and external memeory 
    * interface and more. All these will be done by SystemInit. This 
    * function can be found inside the system_<device>.c file.
    */
   ldr   r0, =SystemInit
   mov   lr, pc
   bx    r0


#if (RUN_FROM_FLASH == 1)
   /*
    * Copy the initialized data of the ".data" segment
    * from the flash to the are in the ram.
    */
   ldr   r1, =_etext
   ldr   r2, =_sdata
   ldr   r3, =_edata

data_loop:
   cmp   r2, r3
   ldrlo r0, [r1], #4
   strlo r0, [r2], #4
   blo   data_loop
#endif

   
   /*
    * Clear the ".bss" segment.
    */
   ldr   r1, =_sbss
   ldr   r2, =_ebss
   ldr   r3, =0

bss_loop:
   cmp   r1, r2
   strne r3, [r1], #+4
   bne   bss_loop
   
   
   /*
    * Set exception stack pointers and disable interrupts.
    */
   ldr   r0, =_efiq_stack
   msr   CPSR_c, #ARM_MODE_FIQ | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT
   mov   r13, r0
   
   ldr   r0, =_eirq_stack
   msr   CPSR_c, #ARM_MODE_IRQ | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT
   mov   r13, r0
   
   ldr   r0, =_eabort_stack
   msr   CPSR_c, #ARM_MODE_ABORT | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT
   mov   r13, r0
   
   ldr   r0, =_eundef_stack
   msr   CPSR_c, #ARM_MODE_UNDEF | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT
   mov   r13, r0

   ldr   r0, =_esvc_stack
   msr   CPSR_c, #ARM_MODE_SVC | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT
   mov   r13, r0
   
   ldr   r0, =_esys_stack
   msr   CPSR_c, #ARM_MODE_SYS | ARM_CPSR_I_BIT | ARM_CPSR_F_BIT
   mov   r13, r0
   
   
   /*
    * And now the main function can be called.
    * Scotty, energie...
    */       
   ldr   r0, =main
   mov   lr, pc
   bx    r0

   
   /*
    * In case there are problems with the
    * "warp drive", stop here.
    */
end:
   nop
   b     end
   
   
/***************************************************************************/
/*  Define the default weak exception handler here                         */
/***************************************************************************/

/*
 * Default undefined handler
 */
UndefHandler:
   b  UndefHandler

/*
 * Default software interrupt handler
 */
SWIHandler:
   b  SWIHandler

/*
 * Default prefetch abort handler
 */
PAbortHandler:
   b  PAbortHandler

/*
 * Default data abort handler
 */
DAbortHandler:
   b  DAbortHandler

/*
 * Default data abort handler
 */
RESHandler:
   b  RESHandler

/*
 * Default IRQ handler
 */
IRQHandler:
   b  IRQHandler
   
/*
 * Default FIQ handler
 */
FIQHandler:
   b  FIQHandler
   
   .weak UndefHandler, SWIHandler, PAbortHandler, DAbortHandler, RESHandler, IRQHandler, FIQHandler
   
/*** EOF ***/
   

 
 
    
