/****************************************************************************
*  Copyright (c) 2011 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  18.04.2011  mifi  First Version for the EIR board.
*                    The REMAP functionality is based on an Atmel example.
****************************************************************************/
#define __SYSTEM_EIR_C__

#include <stdint.h>
#include "at91sam7se512.h"

/*=========================================================================*/
/*  DEFINE: All extern Data                                                */
/*=========================================================================*/
/*
 * The next data are defined by the linker script.
 */
extern unsigned long _stext;

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

#define NUTMEM_SDRAM_BASE  0x20000000
#define NUTMEM_SDRAM_TWR   2
#define NUTMEM_SDRAM_TRC   4
#define NUTMEM_SDRAM_TRP   4
#define NUTMEM_SDRAM_TRCD  2
#define NUTMEM_SDRAM_TRAS  3
#define NUTMEM_SDRAM_TXSR  4

#define SDRAMC_TWR_LSB  7
#define SDRAMC_TRC_LSB  11
#define SDRAMC_TRP_LSB  15
#define SDRAMC_TRCD_LSB 19
#define SDRAMC_TRAS_LSB 23
#define SDRAMC_TXSR_LSB 27


#define _BV(bit)        (1 << (bit))

#define PA23_NWR1_B     23
#define PA24_SDA10_B    24
#define PA25_SDCKE_B    25
#define PA26_NCS1_B     26
#define PA27_SDWE_B     27
#define PA28_CAS_B      28
#define PA29_RAS_B      29

#define PB0_A0_B        0
#define PB2_A2_B        2
#define PB3_A3_B        3
#define PB4_A4_B        4
#define PB5_A5_B        5
#define PB6_A6_B        6
#define PB7_A7_B        7
#define PB8_A8_B        8
#define PB9_A9_B        9
#define PB10_A10_B      10
#define PB11_A11_B      11
#define PB13_A13_B      13
#define PB14_A14_B      14
#define PB15_A15_B      15
#define PB16_A16_B      16
#define PB17_A17_B      17

#define PC0_D0_A        0
#define PC1_D1_A        1
#define PC2_D2_A        2
#define PC3_D3_A        3
#define PC4_D4_A        4
#define PC5_D5_A        5
#define PC6_D6_A        6
#define PC7_D7_A        7
#define PC8_D8_A        8
#define PC9_D9_A        9
#define PC10_D10_A      10
#define PC11_D11_A      11
#define PC12_D12_A      12
#define PC13_D13_A      13
#define PC14_D14_A      14
#define PC15_D15_A      15


/*
 * Remap types:
 *   BOARD_FLASH, Flash is mirrored in the remap zone.
 *   BOARD_RAM, RAM is mirrored in the remap zone.
 */
#define BOARD_FLASH     0
#define BOARD_RAM       1

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  GetRemap                                                               */
/*                                                                         */
/*  Check if the IRAM is still remaped.                                    */ 
/***************************************************************************/
static uint8_t GetRemap (void)
{
   volatile uint32_t *remap = (volatile uint32_t *) 0;
   volatile uint32_t *ram   = (volatile uint32_t *) AT91C_ISRAM;
   uint32_t           temp;

   /* 
    * Try to write in 0 and see if this affects the RAM
    */
   temp = *ram;    
   *ram = temp + 1;
   
   if (*remap == *ram) 
   {
      *ram = temp;
      return(BOARD_RAM);
   }
   else 
   {
      *ram = temp;
      return(BOARD_FLASH);
   }
} /* GetRemap */

/***************************************************************************/
/*  Remap                                                                  */
/*                                                                         */
/*  Check if we must REMAP the IRAM to the Flash.                          */
/***************************************************************************/
static void Remap (void)
{
   if (GetRemap() != BOARD_RAM) 
   {
      AT91C_BASE_MC->MC_RCR = AT91C_MC_RCB;
   }
} /* Remap */

/***************************************************************************/
/*  EnableSDRAM                                                            */
/*                                                                         */
/*  If configured and only if we are running from flash.                   */
/*  In case we are running from RAM, the SDRAM is still                    */
/*  configured by the debugger.                                            */
/*                                                                         */
/***************************************************************************/
static void EnableSDRAM (void)
{
#if defined(NUTMEM_SDRAM_BASE) && (RUN_FROM_FLASH == 1)

   uint32_t  Value;
   uint32_t *pSRAM;
   /* 
    * Enable SDRAM control at PIO A. 
    */
   Value = ( _BV(PA23_NWR1_B)  | 
             _BV(PA24_SDA10_B) | 
             _BV(PA25_SDCKE_B) | 
             _BV(PA26_NCS1_B)  | 
             _BV(PA27_SDWE_B)  | 
             _BV(PA28_CAS_B)   | 
             _BV(PA29_RAS_B) );
    
   AT91C_BASE_PIOA->PIO_BSR = Value;
   AT91C_BASE_PIOA->PIO_PDR = Value;
   
   /* 
    * Enable address bus (A0, A2-A11, A13-A17) at PIO B. 
    */
   Value = ( _BV(PB0_A0_B)   | 
             _BV(PB2_A2_B)   | 
             _BV(PB3_A3_B)   | 
             _BV(PB4_A4_B)   | 
             _BV(PB5_A5_B)   | 
             _BV(PB6_A6_B)   | 
             _BV(PB7_A7_B)   | 
             _BV(PB8_A8_B)   | 
             _BV(PB9_A9_B)   | 
             _BV(PB10_A10_B) | 
             _BV(PB11_A11_B) | 
             _BV(PB13_A13_B) | 
             _BV(PB14_A14_B) | 
             _BV(PB15_A15_B) | 
             _BV(PB16_A16_B) | 
             _BV(PB17_A17_B) );
    
   AT91C_BASE_PIOB->PIO_BSR = Value;
   AT91C_BASE_PIOB->PIO_PDR = Value;
   
   /* 
    * Enable 16 bit data bus at PIO C. 
    */
   Value = ( _BV(PC0_D0_A)   |
             _BV(PC1_D1_A)   |
             _BV(PC2_D2_A)   |
             _BV(PC3_D3_A)   |
             _BV(PC4_D4_A)   |
             _BV(PC5_D5_A)   |
             _BV(PC6_D6_A)   |
             _BV(PC7_D7_A)   |
             _BV(PC8_D8_A)   |
             _BV(PC9_D9_A)   |
             _BV(PC10_D10_A) |
             _BV(PC11_D11_A) |
             _BV(PC12_D12_A) |
             _BV(PC13_D13_A) |
             _BV(PC14_D14_A) |
             _BV(PC15_D15_A) );
    
   AT91C_BASE_PIOC->PIO_ASR = Value;
   AT91C_BASE_PIOC->PIO_PDR = Value; 
    
   /* 
    * Enable SDRAM chip select. 
    */
   AT91C_BASE_EBI->EBI_CSA = AT91C_EBI_CS1A;

   /* 
    * Set SDRAM characteristics in configuration register.
    * Hard coded values for MT48LC32M16A2 with 48MHz CPU. 
    */
   Value = ( AT91C_SDRC_NC_10                       | 
             AT91C_SDRC_NR_13                       | 
             AT91C_SDRC_NB_4_BANKS                  | 
             AT91C_SDRC_CAS_2                       |
             (NUTMEM_SDRAM_TWR << SDRAMC_TWR_LSB)   |
             (NUTMEM_SDRAM_TRC << SDRAMC_TRC_LSB)   |
             (NUTMEM_SDRAM_TRP << SDRAMC_TRP_LSB)   |
             (NUTMEM_SDRAM_TRCD << SDRAMC_TRCD_LSB) |
             (NUTMEM_SDRAM_TRAS << SDRAMC_TRAS_LSB) |
             (NUTMEM_SDRAM_TXSR << SDRAMC_TXSR_LSB) );  
    
   AT91C_BASE_SDRC->SDRC_CR = Value;
   
   /* 200us delay minimum.  */
   for(Value=0; Value<0x2000; Value++);
   
   /* 
    * Load SDRAM base address. 
    */
   pSRAM = (uint32_t*)NUTMEM_SDRAM_BASE;
   
   /* 
    * Issue 16 bit SDRAM command: NOP. 
    */
   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_NOP_CMD);
   pSRAM[0] = 0;
   
   /* 
    * Issue 16 bit SDRAM command: Precharge all. 
    */
   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_PRCGALL_CMD);
   pSRAM[0] = 0;
    
   /* 200us delay minimum.  */
   for(Value=0; Value<0x2000; Value++);
   
   /* 
    * Issue 8 auto-refresh cycles. 
    */
   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_RFSH_CMD);
   pSRAM[0] = 0;

   /* 
    * Issue 16 bit SDRAM command: Set mode register. 
    */
   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_LMR_CMD);
   pSRAM[20] = 0xCAFEDEDE;

   /* 
    * Set refresh rate count. 
    */
   AT91C_BASE_SDRC->SDRC_TR = 384;

   /* 
    * Issue 16 bit SDRAM command: Normal mode. 
    */
   AT91C_BASE_SDRC->SDRC_MR = (AT91C_SDRC_DBW_16_BITS | AT91C_SDRC_MODE_NORMAL_CMD);
   pSRAM[0] = 0;
   
#endif   
} /* EnableSDRAM */

/***************************************************************************/
/*  InitPLL                                                                */
/*                                                                         */
/*  Here the main clock will be initialized.                               */
/***************************************************************************/
static void InitPLL (void)
{
   /*
    * Enable the main oscillator. Set startup time of 0x40 * 8 slow 
    * clock cycles and wait until oscillator is stabilized.
    */
   AT91C_BASE_PMC->PMC_MOR = (0x40 << 8) | AT91C_CKGR_MOSCEN;
   while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MOSCS));

   /*
    * Set PLL:
    * PLLfreq = crystal / divider * (multiplier + 1)
    * Wait 28 clock cycles until PLL is locked.
    */
   AT91C_BASE_PMC->PMC_PLLR = (72 << 16) | (28 << 8) | (14);
   while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCK));

   /* 
    * Wait for the master clock if it was already initialized 
    */
   while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY));

   /*
    * Switch to slow clock with prescaler.
    */
   AT91C_BASE_PMC->PMC_MCKR = AT91C_PMC_PRES_CLK_2 | AT91C_PMC_CSS_SLOW_CLK;
   while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY));

   /*
    * Switch to PLL clock.
    */
   AT91C_BASE_PMC->PMC_MCKR |= AT91C_PMC_CSS_PLL_CLK;
   while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY));
   
} /* InitPLL */


/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  SystemInit                                                             */
/***************************************************************************/
void SystemInit (void)
{      
   /*
    * Configure flash, 2 read and 3 write cycles.
    */
   AT91C_BASE_EFC0->EFC_FMR = AT91C_MC_FWS_2FWS;
   AT91C_BASE_EFC1->EFC_FMR = AT91C_MC_FWS_2FWS;

   /*
    * Disable watchdog
    */
   AT91C_BASE_WDTC->WDTC_WDMR = AT91C_WDTC_WDDIS;
   
   /*
    * Disable all interrupts. Useful for debugging w/o target reset.
    */
   AT91C_BASE_AIC->AIC_EOICR = 0;
   AT91C_BASE_AIC->AIC_IDCR  = 0;
   
   /*
    * Init the main clock
    */
   InitPLL();
   
   
   /*
    * Enable SDRAM interface.
    */
   EnableSDRAM();    
    
   /*
    * Enable external reset key.
    */
   AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
   
   
   /*
    * Copy exception vectors if we running from SDRAM
    */
   if (0x20000000 == (uint32_t)&_stext)
   {
      volatile uint32_t *pSrc  = (uint32_t*)0x20000000;
      volatile uint32_t *pDest = (uint32_t*)0x00200000;
      uint32_t            Index;
   
      /* Copy vectors from SDRAm to IRAM */ 
      for(Index=0; Index<(0x40/sizeof(uint32_t)); Index++)
      {
         *pDest++ = *pSrc++;
      }
   
      Remap();
   }
    
} /* SystemInit */

/*** EOF ***/
